==> echosvg-anim-1.2.5-sources.jar: io/sf/carte/echosvg/anim/values/AnimatableLengthOrIdentValue.java <==
/*

   See the NOTICE file distributed with this work for additional
   information regarding copyright ownership.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 */
package io.sf.carte.echosvg.anim.values;

import org.w3c.dom.svg.SVGLength;

import io.sf.carte.echosvg.anim.dom.AnimationTarget;

/**
 * An SVG length-or-identifier value in the animation system.
 *
 * @author <a href="mailto:cam%40mcc%2eid%2eau">Cameron McCormack</a>
 * @author For later modifications, see Git history.
 * @version $Id$
 */
public class AnimatableLengthOrIdentValue extends AnimatableLengthValue {

	/**
	 * Whether this value is an identifier.
	 */
	protected boolean isIdent;

	/**
	 * The identifier.
	 */
	protected String ident;

	/**
	 * Creates a new, uninitialized AnimatableLengthOrIdentValue.
	 */
	protected AnimatableLengthOrIdentValue(AnimationTarget target) {
		super(target);
	}

	/**
	 * Creates a new AnimatableLengthOrIdentValue for a length value.
	 */
	public AnimatableLengthOrIdentValue(AnimationTarget target, short type, float v, short pcInterp) {
		super(target, type, v, pcInterp);
	}

	/**
	 * Creates a new AnimatableLengthOrIdentValue for an identifier value.
	 */
	public AnimatableLengthOrIdentValue(AnimationTarget target, String ident) {
		super(target);
		this.ident = ident;
		this.isIdent = true;
	}

	/**
	 * Returns whether this value is an identifier or a length.
	 */
	public boolean isIdent() {
		return isIdent;
	}

	/**
	 * Returns the identifier.
	 */
	public String getIdent() {
		return ident;
	}

	/**
	 * Returns whether two values of this type can have their distance computed, as
	 * needed by paced animation.
	 */
	@Override
	public boolean canPace() {
		return false;
	}

	/**
	 * Returns the absolute distance between this value and the specified other
	 * value.
	 */
	@Override
	public float distanceTo(AnimatableValue other) {
		return 0f;
	}

	/**
	 * Returns a zero value of this AnimatableValue's type.
	 */
	@Override
	public AnimatableValue getZeroValue() {
		return new AnimatableLengthOrIdentValue(target, SVGLength.SVG_LENGTHTYPE_NUMBER, 0f, percentageInterpretation);
	}

	/**
	 * Returns the CSS text representation of the value.
	 */
	@Override
	public String getCssText() {
		if (isIdent) {
			return ident;
		}
		return super.getCssText();
	}

	/**
	 * Performs interpolation to the given value.
	 */
	@Override
	public AnimatableValue interpolate(AnimatableValue result, AnimatableValue to, float interpolation,
			AnimatableValue accumulation, int multiplier) {
		AnimatableLengthOrIdentValue res;
		if (result == null) {
			res = new AnimatableLengthOrIdentValue(target);
		} else {
			res = (AnimatableLengthOrIdentValue) result;
		}

		if (to == null) {
			if (isIdent) {
				res.hasChanged = !res.isIdent || !res.ident.equals(ident);
				res.ident = ident;
				res.isIdent = true;
			} else {
				short oldLengthType = res.lengthType;
				float oldLengthValue = res.lengthValue;
				short oldPercentageInterpretation = res.percentageInterpretation;
				super.interpolate(res, to, interpolation, accumulation, multiplier);
				if (res.lengthType != oldLengthType || res.lengthValue != oldLengthValue
						|| res.percentageInterpretation != oldPercentageInterpretation) {
					res.hasChanged = true;
				}
			}
		} else {
			AnimatableLengthOrIdentValue toValue = (AnimatableLengthOrIdentValue) to;
			if (isIdent || toValue.isIdent) {
				if (interpolation >= 0.5) {
					if (res.isIdent != toValue.isIdent || res.lengthType != toValue.lengthType
							|| res.lengthValue != toValue.lengthValue
							|| res.isIdent && toValue.isIdent && !toValue.ident.equals(ident)) {
						res.isIdent = toValue.isIdent;
						res.ident = toValue.ident;
						res.lengthType = toValue.lengthType;
						res.lengthValue = toValue.lengthValue;
						res.hasChanged = true;
					}
				} else {
					if (res.isIdent != isIdent || res.lengthType != lengthType || res.lengthValue != lengthValue
							|| res.isIdent && isIdent && !res.ident.equals(ident)) {
						res.isIdent = isIdent;
						res.ident = ident;
						res.ident = ident;
						res.lengthType = lengthType;
						res.hasChanged = true;
					}
				}
			} else {
				super.interpolate(res, to, interpolation, accumulation, multiplier);
			}
		}

		return res;
	}

}

==> echosvg-anim-2.4-sources.jar: io/sf/carte/echosvg/anim/values/AnimatableLengthOrIdentValue.java <==
/*

   See the NOTICE file distributed with this work for additional
   information regarding copyright ownership.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 */
package io.sf.carte.echosvg.anim.values;

import org.w3c.css.om.unit.CSSUnit;

import io.sf.carte.echosvg.anim.dom.AnimationTarget;

/**
 * An SVG length-or-identifier value in the animation system.
 *
 * @author <a href="mailto:cam%40mcc%2eid%2eau">Cameron McCormack</a>
 * @author For later modifications, see Git history.
 * @version $Id$
 */
public class AnimatableLengthOrIdentValue extends AnimatableLengthValue {

	/**
	 * Whether this value is an identifier.
	 */
	protected boolean isIdent;

	/**
	 * The identifier.
	 */
	protected String ident;

	/**
	 * Creates a new, uninitialized AnimatableLengthOrIdentValue.
	 */
	protected AnimatableLengthOrIdentValue(AnimationTarget target) {
		super(target);
	}

	/**
	 * Creates a new AnimatableLengthOrIdentValue for a length value.
	 */
	public AnimatableLengthOrIdentValue(AnimationTarget target, short type, float v, short pcInterp) {
		super(target, type, v, pcInterp);
	}

	/**
	 * Creates a new AnimatableLengthOrIdentValue for an identifier value.
	 */
	public AnimatableLengthOrIdentValue(AnimationTarget target, String ident) {
		super(target);
		this.ident = ident;
		this.isIdent = true;
	}

	/**
	 * Returns whether this value is an identifier or a length.
	 */
	public boolean isIdent() {
		return isIdent;
	}

	/**
	 * Returns the identifier.
	 */
	public String getIdent() {
		return ident;
	}

	/**
	 * Returns whether two values of this type can have their distance computed, as
	 * needed by paced animation.
	 */
	@Override
	public boolean canPace() {
		return false;
	}

	/**
	 * Returns the absolute distance between this value and the specified other
	 * value.
	 */
	@Override
	public float distanceTo(AnimatableValue other) {
		return 0f;
	}

	/**
	 * Returns a zero value of this AnimatableValue's type.
	 */
	@Override
	public AnimatableValue getZeroValue() {
		return new AnimatableLengthOrIdentValue(target, CSSUnit.CSS_NUMBER, 0f, percentageInterpretation);
	}

	/**
	 * Returns the CSS text representation of the value.
	 */
	@Override
	public String getCssText() {
		if (isIdent) {
			return ident;
		}
		return super.getCssText();
	}

	/**
	 * Performs interpolation to the given value.
	 */
	@Override
	public AnimatableValue interpolate(AnimatableValue result, AnimatableValue to, float interpolation,
			AnimatableValue accumulation, int multiplier) {
		AnimatableLengthOrIdentValue res;
		if (result == null) {
			res = new AnimatableLengthOrIdentValue(target);
		} else {
			res = (AnimatableLengthOrIdentValue) result;
		}

		if (to == null) {
			if (isIdent) {
				res.hasChanged = !res.isIdent || !res.ident.equals(ident);
				res.ident = ident;
				res.isIdent = true;
			} else {
				short oldLengthType = res.lengthType;
				float oldLengthValue = res.lengthValue;
				short oldPercentageInterpretation = res.percentageInterpretation;
				super.interpolate(res, to, interpolation, accumulation, multiplier);
				if (res.lengthType != oldLengthType || res.lengthValue != oldLengthValue
						|| res.percentageInterpretation != oldPercentageInterpretation) {
					res.hasChanged = true;
				}
			}
		} else {
			AnimatableLengthOrIdentValue toValue = (AnimatableLengthOrIdentValue) to;
			if (isIdent || toValue.isIdent) {
				if (interpolation >= 0.5) {
					if (res.isIdent != toValue.isIdent || res.lengthType != toValue.lengthType
							|| res.lengthValue != toValue.lengthValue
							|| res.isIdent && toValue.isIdent && !toValue.ident.equals(ident)) {
						res.isIdent = toValue.isIdent;
						res.ident = toValue.ident;
						res.lengthType = toValue.lengthType;
						res.lengthValue = toValue.lengthValue;
						res.hasChanged = true;
					}
				} else {
					if (res.isIdent != isIdent || res.lengthType != lengthType || res.lengthValue != lengthValue
							|| res.isIdent && isIdent && !res.ident.equals(ident)) {
						res.isIdent = isIdent;
						res.ident = ident;
						res.ident = ident;
						res.lengthType = lengthType;
						res.hasChanged = true;
					}
				}
			} else {
				super.interpolate(res, to, interpolation, accumulation, multiplier);
			}
		}

		return res;
	}

}
